#include "ubinstall-gtk.h"


// void on_open_documentation_confirmation(GtkWidget *self, char *link);
// void on_open_documentation_confirmation(GtkWidget *self, char *link){
//     if (main_config.always_open_documentation==0){
//         GtkBuilder *builder = gtk_builder_new_from_resource(ui_glade_path_documentation);
//         documentation_confirmation_window *window = malloc(sizeof(documentation_confirmation_window));
//         window->Window = yon_gtk_builder_get_widget(builder,"helpConfirmationWindow");
//         window->AcceptButton = yon_gtk_builder_get_widget(builder,"ReadHelpButton");
//         window->CloseButton = yon_gtk_builder_get_widget(builder,"CancelHelpButton");
//         window->HeaderLabel = yon_gtk_builder_get_widget(builder,"webHeaderNameLabel");
//         window->AlwaysOpenCheck = yon_gtk_builder_get_widget(builder,"AlwaysOpenDocumentationCheckbox");
//         gtk_label_set_text(GTK_LABEL(window->HeaderLabel),TITLE_LABEL);
//         gtk_widget_show_all(window->Window);
//         g_signal_connect(G_OBJECT(window->CloseButton),"clicked",G_CALLBACK(on_subwindow_close),NULL);
//         g_signal_connect(G_OBJECT(window->AcceptButton),"clicked",G_CALLBACK(yon_open_browser),yon_char_new(link));
//         g_signal_connect(G_OBJECT(window->AcceptButton),"clicked",G_CALLBACK(on_subwindow_close),NULL);
//
//      
//     } else {
//         yon_open_browser(self,link);
//     }
// }

// void on_link(GtkWidget *self, char* uri);
// void on_link(GtkWidget *self, char* uri){
//     gtk_widget_destroy(self);
//     on_open_documentation_confirmation(self,uri);
// }

// void on_about(GtkWidget *);
// void on_about(GtkWidget *){
//     GtkBuilder *builder=gtk_builder_new_from_resource(ui_glade_path_about);
//     GtkWidget *window=yon_gtk_builder_get_widget(builder,"AboutWindow");
//     GtkWidget *title=yon_gtk_builder_get_widget(builder,"headerAboutTopic");
//     GtkWidget *hideButtonBox=yon_gtk_builder_get_widget(builder,"buttonBoxHide");
//     gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(window),version_application);
//     gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(window),TITLE_LABEL);
//     gtk_about_dialog_set_logo_icon_name(GTK_ABOUT_DIALOG(window),yon_char_append("com.ublinux.",LocaleName));
//     gtk_window_set_icon_name(GTK_WINDOW(window),yon_char_append("com.ublinux.",LocaleName));
//     gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(window),LocaleName);
//     gtk_label_set_text(GTK_LABEL(title),TITLE_LABEL);
//     g_signal_connect(G_OBJECT(window),"activate-link",G_CALLBACK(on_link),WIKI_LINK);
//     gtk_widget_set_visible(hideButtonBox,0);
//     gtk_widget_destroy(hideButtonBox);
//     gtk_widget_show(window);
// }
